from pathlib import Path
from zen_generator.generators.asyncapi import generate_asyncapi_from_files, create_async_api_content


def test_create_async_api_content() -> None:
    content = {"asyncapi": "2.0.0", "info": {"title": "Test API", "version": "1.0.0"}}
    models_schema = {"TestModel": {"type": "object", "properties": {"id": {"type": "integer"}}}}
    functions_docstring = "This is a test function."
    functions_parsed = {
        "test_function": {"description": "A test function", "request": {"payload": {}}, "response": {"payload": {}}}
    }
    output = create_async_api_content("Test API", models_schema, functions_docstring, functions_parsed)
    assert output is not None
    assert "asyncapi" in output
    assert output["info"]["title"] == "Test API"


def test_generate_asyncapi_from_files(tmp_path) -> None:
    models_file = Path("models.py")
    functions_file = Path("functions.py")
    output_file = tmp_path / "output_path.yaml"
    app_name = "TestApp"
    assert not output_file.exists()
    generate_asyncapi_from_files(models_file, functions_file, output_file, app_name)
    assert output_file.exists()
    with output_file.open() as f:
        content = f.read()

    # check that the content contains the expected functions and models
    assert "get_attachments_from_utd" in content
    assert "empty" in content
    assert "generate_sync_task" in content
    assert "generate_iiacc_task" in content
    assert "generate_iva_sync_task" in content
    assert "Descrizione metodo get_attachments_from_utd" in content

    assert "TaskAttachment" in content
    assert "UserTaxDeclarationInfo" in content
    assert "Mida4TaskEnvironmentChoices" in content



from __future__ import annotations

from pathlib import Path

import pytest

from zen_generator.cli import fastapi, pure_python
from zen_generator.core.ast_utils import generate_code_from_ast, get_component_schemas
from zen_generator.core.exception import InvalidFile
from zen_generator.core.io import load_yaml
from zen_generator.generators.python import Generator


def test_source_path_yaml_is_valid() -> None:
    file: Path = Path("async_api_test.yaml")
    assert load_yaml(file)


def test_source_path_yaml_directory_is_not_valid() -> None:
    file: Path = Path("./")
    with pytest.raises(InvalidFile, match="The source is a directory"):
        load_yaml(file)


def test_source_path_yaml_not_found_is_not_valid() -> None:
    file: Path = Path("./fake_path")
    with pytest.raises(InvalidFile):
        load_yaml(file)


def test_generate_model() -> None:
    source: Path = Path("./test_models.yaml")
    source_content = load_yaml(source)
    component_schemas = get_component_schemas(source_content) or {}

    generator = Generator.pure_python_generator()
    generator.component_schemas = component_schemas
    generator.generate_models_ast()

    model = generator.models_ast
    py = generate_code_from_ast(model)

    aspect_result = """from __future__ import annotations

from typing import TypedDict

from utils.enums import Choices


class TaskAttachment(TypedDict):
    name: str
    kind: str


class UserTaxDeclarationInfo(TypedDict):
    utd_id: int | None
    full_environment: bool


class Mida4TaskEnvironmentChoices(Choices):
    pass


class FooBar(TypedDict):
    env: str | None
    baz: UserTaxDeclarationInfo | list[bool] | int
    foo: str | object
"""

    assert py == aspect_result


def test_generate_functions() -> None:
    source = Path("./test_functions.yaml")
    app_name = "Fake"
    generator = Generator.pure_python_generator()

    generator.load_asyncapi_content(source)
    generator.generate_function_ast(app_name)
    py = generate_code_from_ast(generator.functions_ast)

    aspect_result = """\"\"\"Test API description\"\"\"

from __future__ import annotations

import logging
from typing import TypedDict

from utils.enums import Choices

logger = logging.getLogger("Fake")


def get_attachments_from_utd(utd_id: int | str | TaskAttachment, kinds: list[str]) -> list[TaskAttachment]: ...


def generate_sync_task(
    utd_info: UserTaxDeclarationInfo, sequence: int | None, skip_create_f24s: bool
) -> int | None: ...


def generate_iiacc_task(
    utd_info: UserTaxDeclarationInfo, fiscal_elements: list[object] | None
) -> int | str | TaskAttachment | None: ...


def generate_iva_sync_task(
    user_id: int, ref_year: int, environment: Mida4TaskEnvironmentChoices, custom_actions: object | None
) -> None: ...
"""

    assert py == aspect_result


def test_generate_functions_and_model_from_yaml(tmp_path) -> None:
    models_file = tmp_path / "models.py"
    functions_file = tmp_path / "functions.py"
    source = Path("./test.yaml")
    app_name = "Fake"

    assert not models_file.exists() and not functions_file.exists()

    pure_python(source, models_file, functions_file, app_name)

    assert models_file.exists() and functions_file.exists()


## FASTAPI


def test_generate_fastapi_models() -> None:
    source: Path = Path("./test_models.yaml")
    source_content = load_yaml(source)
    component_schemas = get_component_schemas(source_content) or {}

    generator = Generator.fastapi_generator()
    generator.component_schemas = component_schemas
    generator.generate_models_ast()

    model = generator.models_ast
    py = generate_code_from_ast(model)

    aspect_result = """from __future__ import annotations

from fastapi import FastAPI


class TaskAttachment(BaseModel):
    name: str
    kind: str


class UserTaxDeclarationInfo(BaseModel):
    utd_id: int | None
    full_environment: bool


class Mida4TaskEnvironmentChoices(BaseModel):
    pass


class FooBar(BaseModel):
    env: str | None
    baz: UserTaxDeclarationInfo | list[bool] | int
    foo: str | object
"""

    assert py == aspect_result


def test_generate_endpoints() -> None:
    source = Path("./test_functions.yaml")
    app_name = "Fake"
    generator = Generator.fastapi_generator()

    generator.load_asyncapi_content(source)
    generator.generate_function_ast(app_name)
    py = generate_code_from_ast(generator.functions_ast)

    aspect_result = """\"\"\"Test API description\"\"\"

from __future__ import annotations

import logging

from fastapi import FastAPI

app = FastAPI()
logger = logging.getLogger("Fake")


@app.get("/get_attachments_from_utd")
def get_attachments_from_utd(utd_id: int | str | TaskAttachment, kinds: list[str]) -> list[TaskAttachment]: ...


@app.get("/generate_sync_task")
def generate_sync_task(
    utd_info: UserTaxDeclarationInfo, sequence: int | None, skip_create_f24s: bool
) -> int | None: ...


@app.get("/generate_iiacc_task")
def generate_iiacc_task(
    utd_info: UserTaxDeclarationInfo, fiscal_elements: list[object] | None
) -> int | str | TaskAttachment | None: ...


@app.get("/generate_iva_sync_task")
def generate_iva_sync_task(
    user_id: int, ref_year: int, environment: Mida4TaskEnvironmentChoices, custom_actions: object | None
) -> None: ...
"""
    assert py == aspect_result


def test_generate_fastapi_from_yaml(tmp_path) -> None:
    models_file = tmp_path / "models.py"
    functions_file = tmp_path / "functions.py"
    source = Path("./test.yaml")
    app_name = "Fake"

    assert not models_file.exists() and not functions_file.exists()

    fastapi(source, models_file, functions_file, app_name)

    assert models_file.exists() and functions_file.exists()



from __future__ import annotations

from pathlib import Path

from zen_generator.core.ast_utils import generate_component_schemas
from zen_generator.core.io import parse_python_file_to_ast
from zen_generator.core.parsing import function_content_reader
from zen_generator.generators.asyncapi import create_async_api_content


def test_generate_yaml_from_functions_and_models() -> None:
    source_models: Path = Path("./models.py")
    source_functions: Path = Path("./functions.py")
    app_name: str = "Fake"

    models_ast = parse_python_file_to_ast(source_models)
    models_schema = generate_component_schemas(models_ast)

    functions_ast = parse_python_file_to_ast(source_functions)
    functions_docstring, functions_parsed = function_content_reader(functions_ast)

    async_api_content = create_async_api_content(app_name, models_schema, functions_docstring, functions_parsed)
    assert async_api_content



"""This module contains utilities for formatting Python code."""

from __future__ import annotations

import subprocess
import tempfile
from pathlib import Path


def format_python_code(code: str) -> str:
    """Format python code using Ruff.

    Format a given string of python code using the Ruff formatter.

    Args:
        code (str): Python code to format

    Returns:
        str: Formatted python code if successful, original code otherwise.
    """
    with tempfile.NamedTemporaryFile(suffix=".py", mode="w+", delete=False) as tmp:
        tmp.write(code)
        tmp_path = Path(tmp.name)

    try:
        # https://docs.astral.sh/ruff/formatter/#line-breaks
        # ruff check --select I --fix
        subprocess.run(
            ["ruff", "check", "--select", "I", "--fix", str(tmp_path)],
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        # ruff format
        subprocess.run(
            ["ruff", "format", str(tmp_path)],
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        formatted_code: str = tmp_path.read_text()
        return formatted_code
    except subprocess.CalledProcessError:
        return code
    finally:
        tmp_path.unlink(missing_ok=True)



from __future__ import annotations

from ast import ClassDef, Constant, Expr, FunctionDef, Load, Name, Pass, arg, arguments


from zen_generator.core.ast_utils import generate_code_from_ast
from zen_generator.core.formatting import format_python_code


def test_generate_code_from_ast_with_function() -> None:
    func_def = FunctionDef(
        name="test_function",
        args=arguments(
            posonlyargs=[],
            args=[
                arg(arg="x", annotation=Name(id="str", ctx=Load())),
                arg(arg="y", annotation=Name(id="int", ctx=Load())),
            ],
            kwonlyargs=[],
            kw_defaults=[],
            defaults=[],
        ),
        body=[Pass()],
        decorator_list=[],
        returns=Name(id="None", ctx=Load()),
    )
    ast_nodes = [func_def]

    result = generate_code_from_ast(ast_nodes)

    expected_code = format_python_code("def test_function(x:str, y:int) -> None:\n    pass\n")
    assert result == expected_code


def test_generate_code_from_ast_with_class() -> None:
    class_def = ClassDef(
        name="TestClass",
        bases=[],
        keywords=[],
        body=[
            FunctionDef(
                name="__init__",
                args=arguments(
                    posonlyargs=[],
                    args=[arg(arg="self", annotation=None)],
                    kwonlyargs=[],
                    kw_defaults=[],
                    defaults=[],
                ),
                body=[Pass()],
                decorator_list=[],
                returns=Name(id="None", ctx=Load()),
            )
        ],
        decorator_list=[],
    )
    ast_nodes = [class_def]

    result = generate_code_from_ast(ast_nodes)

    expected_code = format_python_code("class TestClass:\n    def __init__(self) -> None:\n        pass\n")
    assert result == expected_code


def test_generate_code_from_ast_with_multiple_nodes() -> None:
    func_def = FunctionDef(
        name="test_function",
        args=arguments(
            posonlyargs=[],
            args=[
                arg(arg="x", annotation=Name(id="str", ctx=Load())),
                arg(arg="y", annotation=Name(id="bool", ctx=Load())),
            ],
            kwonlyargs=[],
            kw_defaults=[],
            defaults=[Constant(value=True)],
        ),
        body=[Pass()],
        decorator_list=[],
        returns=Name(id="None", ctx=Load()),
    )
    class_def = ClassDef(
        name="TestClass",
        bases=[],
        keywords=[],
        body=[
            FunctionDef(
                name="__init__",
                args=arguments(
                    posonlyargs=[],
                    args=[arg(arg="self", annotation=None)],
                    kwonlyargs=[],
                    kw_defaults=[],
                    defaults=[],
                ),
                body=[Pass()],
                decorator_list=[],
                returns=Name(id="None", ctx=Load()),
            )
        ],
        decorator_list=[],
    )
    ast_nodes = [func_def, class_def]

    result = generate_code_from_ast(ast_nodes)

    expected_code = format_python_code(
        "def test_function(x:str, y:bool = True) -> None:\n    pass\n\nclass TestClass:\n    def __init__(self) -> None:\n        pass\n"
    )
    assert result == expected_code
